""" version which can be consumed from within the module """
VERSION_STR = "0.0.17"
DESCRIPTION = "command line utilities to help you work with paths"
APP_NAME = "pypathutil"
LOGGER_NAME = "pypathutil"



"""
All configurations for pypathutil
"""
import os

from pytconf import Config, ParamCreator


class ConfigSeparator(Config):
    """
    Parameters for configuring the separator
    """
    separator = ParamCreator.create_str(
        help_string="What is the path separator?",
        default=os.pathsep,
    )


class ConfigOps(Config):
    """
    Which operations to perform on the path
    """
    remove_non_abs = ParamCreator.create_bool(
        help_string="remove non absolute folder elements from the path?",
        default=True,
    )
    remove_non_folders = ParamCreator.create_bool(
        help_string="remove non folder elements from the path?",
        default=True,
    )
    remove_duplicates = ParamCreator.create_bool(
        help_string="remove duplicate elements from the path?",
        default=True,
    )


class ConfigPath(Config):
    """
    The existing path
    """
    path = ParamCreator.create_str(
        help_string="What is the path to work on?",
    )


class ConfigFolder(Config):
    """
    A folder name
    """
    folder = ParamCreator.create_str(
        help_string="A folder to work with"
    )


class ConfigHead(Config):
    """
    Configure whether we need head or tail
    """
    head = ParamCreator.create_bool(
        help_string="add to the head of the path?",
        default=True,
    )



"""
main
"""
import pylogconf.core
from pytconf import register_endpoint, register_main, config_arg_parse_and_launch

from pypathutil import common
from pypathutil.configs import ConfigFolder, ConfigPath, ConfigHead, ConfigSeparator, ConfigOps
from pypathutil.static import DESCRIPTION, APP_NAME, VERSION_STR


@register_endpoint(
    configs=[
        ConfigFolder,
        ConfigPath,
        ConfigHead,
        ConfigSeparator,
        ConfigOps,
    ],
    description="Add two paths together",
)
def add() -> None:
    new_path = common.add(
        path=ConfigPath.path,
        folder=ConfigFolder.folder,
        head=ConfigHead.head,
        separator=ConfigSeparator.separator,
        remove_duplicates=ConfigOps.remove_duplicates,
        remove_non_folders=ConfigOps.remove_non_folders,
        remove_non_abs=ConfigOps.remove_non_abs,
    )
    print(new_path)


@register_endpoint(
    configs=[
        ConfigSeparator,
        ConfigOps,
        ConfigPath,
    ],
    description="Clean a path, removing elements which repeat or are not valid paths",
)
def clean() -> None:
    path = common.clean(
        path=ConfigPath.path,
        separator=ConfigSeparator.separator,
        remove_duplicates=ConfigOps.remove_duplicates,
        remove_non_folders=ConfigOps.remove_non_folders,
        remove_non_abs=ConfigOps.remove_non_abs,
    )
    print(path)


@register_endpoint(
    configs=[
        ConfigOps,
        ConfigSeparator,
        ConfigPath,
        ConfigFolder,
    ],
    description="Remove element from a path",
)
def remove() -> None:
    new_path = common.remove(
        path=ConfigPath.path,
        folder=ConfigFolder.folder,
        separator=ConfigSeparator.separator,
        remove_duplicates=ConfigOps.remove_duplicates,
        remove_non_folders=ConfigOps.remove_non_folders,
        remove_non_abs=ConfigOps.remove_non_abs,
    )
    print(new_path)


@register_main(
    main_description=DESCRIPTION,
    app_name=APP_NAME,
    version=VERSION_STR,
)
def main():
    pylogconf.core.setup()
    config_arg_parse_and_launch()


if __name__ == '__main__':
    main()
